import json
import requests
from regru_cloudapi.utils import Errors


class CloudAPI(object):

    def __init__(self, token=None):
        self.token = token
        self.api_url = 'https://api.cloudvps.reg.ru/v1'
        self.HEADERS = {'Content-Type': 'application/json'}

        if self.token is not None:
            self.HEADERS['Authorization'] = f'Bearer {self.token}'

    def get_tariffs(self):
        data = requests.get(f'{self.api_url}/sizes', headers=self.HEADERS).json()

        return data

    def get_prices(self):
        data = requests.get(f'{self.api_url}/prices', headers=self.HEADERS).json()

        return data

    def get_balance_data(self):
        data = requests.get(f'{self.api_url}/balance_data', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def images(self, param_type):
        Errors(parameter=param_type).check_images()

        params = {'type': param_type}

        data = requests.get(f'{self.api_url}/images', headers=self.HEADERS, params=params).json()
        check_data = Errors(data).check_error()

        return check_data

    def get_ssh_keys(self):
        data = requests.get(f'{self.api_url}/account/keys', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def add_ssh_key(self, name, pkey):
        data_params = {'name': name,
                       'public_key': pkey}

        data = requests.post(f'{self.api_url}/account/keys',
                             headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_ssh_key()

        return check_data

    def rename_ssh_key(self, name, key_id):
        data_params = {'name': name}

        data = requests.put(f'{self.api_url}/account/keys/{key_id}',
                            headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_ssh_key()

        return check_data

    def delete_ssh_key(self, key_id):
        data = requests.delete(f'{self.api_url}/account/keys/{key_id}',
                               headers=self.HEADERS)

        if data.status_code != 204:
            check_data = Errors(data.json()).check_ssh_key()

            return check_data
        else:
            return True

    def ptr(self, domain, ip):
        data_params = {'ptr': domain}

        data = requests.put(f'{self.api_url}/ips/{ip}',
                            headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def get_reglets(self):
        data = requests.get(f'{self.api_url}/reglets', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def create_reglet(self, size, image, name=None, ssh_keys=None, backups=None):
        data_params = {'size': size,
                       'image': image}

        if name is not None:
            data_params['name'] = name

        if ssh_keys is not None:
            data_params['ssh_keys'] = ssh_keys

        if backups is not None:
            data_params['backups'] = backups

        data = requests.post(f'{self.api_url}/reglets',
                             headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def actions(self, reglet_id, action, size=None, image=None, offline=None, name=None):
        Errors(parameter=action).check_actions()

        data_params = {'type': action}

        if action == 'resize':
            if size is not None:
                data_params['size'] = size
            else:
                raise ValueError('Значение size не может быть None')
        elif action == 'rebuild' or action == 'restore':
            if image is not None:
                data_params['image'] = image
            else:
                raise ValueError('Значение image не может быть None')
        elif action == 'clone' or action == 'snapshot':
            if offline is not None:
                data_params['offline'] = offline

            if name is not None:
                data_params['name'] = name

        data = requests.post(f'{self.api_url}/reglets/{reglet_id}/actions',
                             headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def rename_reglet(self, reglet_id, name):
        if name is not None:
            data_params = {'name': name}
        else:
            raise ValueError('Переменная name не может быть None')

        data = requests.put(f'{self.api_url}/reglets/{reglet_id}',
                            headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def delete_reglet(self, reglet_id):
        data = requests.delete(f'{self.api_url}/reglets/{reglet_id}',
                               headers=self.HEADERS)

        if data.status_code != 204:
            check_data = Errors(data.json()).check_error()

            return check_data
        else:
            return True

    def get_snapshots(self):
        data = requests.get(f'{self.api_url}/snapshots', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def delete_snapshot(self, snap_id):
        data = requests.delete(f'{self.api_url}/snapshots/{snap_id}', headers=self.HEADERS)

        if data.status_code != 204:
            check_data = Errors(data.json()).check_error()

            return check_data
        else:
            return True

    def get_additional_ips(self, reglet_id=None, ip=None):
        params = {}

        if reglet_id is not None:
            params['reglet_id'] = reglet_id

        if ip is None:
            data = requests.get(f'{self.api_url}/ips', headers=self.HEADERS, params=params).json()
        else:
            data = requests.get(f'{self.api_url}/ips/{ip}', headers=self.HEADERS).json()

        check_data = Errors(data).check_error()

        return check_data

    def add_additional_ips(self, reglet_id, ipv4_count=None, ipv6_count=None):
        data_params = {}

        if ipv4_count is not None:
            data_params['ipv4_count'] = ipv4_count
        elif ipv6_count is not None:
            data_params['ipv6_count'] = ipv6_count

        if data_params is not None:
            data_params['reglet_id'] = reglet_id

            data = requests.post(f'{self.api_url}/ips', headers=self.HEADERS, data=json.dumps(data_params)).json()
            check_data = Errors(data).check_error()

            return check_data
        else:
            raise ValueError('Не указан ни один из параметров - ipv4_count, ipv6_count')

    def delete_additional_ips(self, ip):
        if ip is not None:
            data = requests.delete(f'{self.api_url}/ips/{ip}', headers=self.HEADERS)

            if data.status_code == 204:
                return True
            else:
                check_data = Errors(data.json()).check_error()
                return check_data
        else:
            raise ValueError('Переменная ip не может быть None')

    def get_info_action(self, action_id):
        data = requests.get(f'{self.api_url}/actions/{action_id}', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def get_vpcs(self):
        data = requests.get(f'{self.api_url}/vpcs', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def get_vpcs_info(self, vpcs_id):
        data = requests.get(f'{self.api_url}/vpcs/{vpcs_id}', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def add_vpcs(self, name):
        data_params = {'name': name}

        data = requests.post(f'{self.api_url}/vpcs', headers=self.HEADERS,
                             data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def rename_vpcs(self, vpcs_id, name):
        data_params = {'name': name}

        data = requests.put(f'{self.api_url}/vpcs/{vpcs_id}',
                            headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def delete_vpcs(self, vpcs_id):
        data = requests.delete(f'{self.api_url}/vpcs/{vpcs_id}', headers=self.HEADERS)

        if data.status_code == 204:
            return True
        else:
            check_data = Errors(data.json()).check_error()
            return check_data

    def get_vpcs_members(self, vpcs_id):
        data = requests.get(f'{self.api_url}/vpcs/{vpcs_id}/members', headers=self.HEADERS).json()
        check_data = Errors(data).check_error()

        return check_data

    def join_vpcs_member(self, reglet_id, vpcs_id):
        data_params = {'resource_id': reglet_id}

        data = requests.post(f'{self.api_url}/vpcs/{vpcs_id}/members',
                             headers=self.HEADERS, data=json.dumps(data_params)).json()
        check_data = Errors(data).check_error()

        return check_data

    def disconnect_vpcs_member(self, reglet_id, vpcs_id):
        data = requests.delete(f'{self.api_url}/vpcs/{vpcs_id}/members/{reglet_id}', headers=self.HEADERS)

        if data.status_code == 204:
            return True
        else:
            check_data = Errors(data.json()).check_error()
            return check_data



class Errors(object):
    def __init__(self, data=None, parameter=None, token=None):
        self.data = data
        self.parameter = parameter
        self.token = token

    def check_error(self):
        if 'error' in self.data:
            if self.data['error'] == 'Authorization required':
                raise ValueError('Ключ API неверный. Проверь ключ API в Личном Кабинете Reg.ru услуги Облачные сервера')
        elif 'action' in self.data:
            if self.data['action'] is None:
                raise KeyError('Задание с таким id не найдено')
        elif 'detail' in self.data:
            detail = self.data['detail']

            if 'is too short' in detail and 'ptr' in detail:
                raise ValueError('Предоставленный домен неверен')
            elif 'does not match' in self.data['detail'] and \
                    '^(?:([0-9]{1,3}\\\\.){3}[0-9]{1,3})' in detail:
                raise ValueError('Предоставленный домен неверен')
            elif 'The server encountered an internal error and was unable to complete your request' in detail:
                raise ValueError('Вероятно был использован неверный id')
            else:
                return self.data
        elif 'code' in self.data:
            code = self.data['code']

            if 'IP_NOT_FOUND' in code:
                raise ValueError('Такой IP-адрес не найден')
            elif 'NO_SUCH_REGLET' in code:
                raise ValueError('Такой сервер отсутствует')
            elif 'NO_SUCH_SNAPSHOT' in code:
                raise ValueError('Такой снэпшот отсутствует')
            elif 'NOT_IMPLEMENTED' in code:
                raise ValueError('Такой id отсутствует')
            elif 'VALIDATION_ERROR' in code and 'data' in self.data:
                if 'image' in self.data['data'][0]:
                    raise ValueError('Проверь значение image')
                elif 'size' in self.data['data'][0]:
                    raise ValueError('Проверь значение size')
                elif 'ssh_key_id' in self.data['data'][0]:
                    raise ValueError('Проверь значение ssh_keys')
                elif self.data['data'][0] == 'id':
                    raise KeyError('Сервер с таким id не существует. Проверь id сервера')
                elif self.data['data'][0] == 'name':
                    raise ValueError('Неверное название сервера')
                else:
                    return self.data
            elif 'SSH_KEY_DOES_NOT_EXIST' in code:
                raise KeyError('SSH-ключ с этим id не существует')
            elif 'RESOURCE_LOCKED' in code:
                raise RuntimeError('Реглет заблокирован, так как задание уже выполняется')
            elif 'RESOURCE_NOT_FOUND' in code:
                raise KeyError('Ресурс не был найден. '
                               'Вероятно был указан неверный id ресурса, либо он не существует')
            elif 'ERROR_IP_COUNT' in code:
                raise KeyError('Ошибка в количестве IP-адресов')
            else:
                return self.data
        else:
            return self.data

    def check_images(self):
        params = ['distribution', 'application', 'snapshot', 'backup']
        if self.parameter not in params:
            raise KeyError('Отсутствующее значение. Используй одно из значений: '
                           'distribution, application, snapshot, backup')

    def check_actions(self):
        params = ['reboot', 'password_reset', 'start', 'stop', 'enable_backups', 'disable_backups', 'resize',
                  'rebuild', 'restore', 'clone', 'snapshot']

        if self.parameter not in params:
            raise KeyError('Такой тип операции отсутстсвует. Используй одно из значений: '
                           'reboot, password_reset, start, stop, enable_backups, disable_backups, resize,'
                           'rebuild, restore, clone, snapshot')

    def check_ssh_key(self):
        if 'detail' in self.data:
            if 'does not match' in self.data['detail']:
                if 'name' in self.data['detail']:
                    raise ValueError('Неверное имя ключа')
                elif 'public_key' in self.data['detail']:
                    raise ValueError('Предоставленный SSH-ключ неверен')
        elif 'code' in self.data:
            if self.data['code'] == 'SSH_KEY_ALREADY_EXIST':
                raise ValueError(f'Предоставленный SSH-ключ уже существует: {self.data}')
            elif self.data['code'] == 'SSH_KEY_DOES_NOT_EXIST':
                raise KeyError(f'SSH-ключ с этим id не существует: {self.data}')
        else:
            return Errors(self.data).check_error()



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))


def read(filename):
    with open(filename, encoding='utf-8') as file:
        return file.read()


setup(
    name='regru_cloudapi',
    description='Reg.ru CloudAPI Python library',
    version='1.2.4',
    url='https://github.com/plvskiy/regru_cloudapi',
    author='Matthew Polovskiy',
    packages=find_packages(),
    python_requires='>=3.6',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    license='MIT',

    classifiers=['Programming Language :: Python :: 3',
                 'License :: OSI Approved :: MIT License']
)


